@@ -769,7 +769,7 @@ class CodeInterface(OptionalAttributes):
         self.channel.redirect_stdout_file = self.redirection_filenames[0]
         self.channel.redirect_stderr_file = self.redirection_filenames[1]
         self.channel.polling_interval_in_milliseconds = self.polling_interval_in_milliseconds
-        self.channel.initialize_mpi = self.initialize_mpi
+        #~ self.channel.initialize_mpi = self.initialize_mpi
         
         self.channel.start()
         
@@ -906,8 +906,6 @@ class CodeInterface(OptionalAttributes):
     def channel_type(self):
         return 'mpi'
     
-
-
     @option(type="boolean", sections=("channel",))
     def initialize_mpi(self):
         """Is MPI initialized in the code or not. Defaults to True if MPI is available"""
